library(class)
library(rpart)

# read the data

rdemo <- read.table("rdemo.dat",header=F,col.names=c("x1","x2","y"))

# determine number of observations and set training and validation sizes

n.total <- nrow(rdemo)
n.train <- 0.60*n.total
n.valid  <- n.total - n.train

# put the data in random order

set.seed(100);
idx <- runif(n.total)
idx <- sort.list(idx)
rdemo <- rdemo[idx,]

# partition the data

partition <- mat.or.vec(n.total,1)
for (i in 1:n.train) partition[i] <- 1

rdemo <- cbind(rdemo,partition)

# run regression
print(" ")
print("regression")

fit <- lm(formula = y ~ x1 + x2, data = rdemo, weights = partition)

#print(fit)

# classify observations

default <- mat.or.vec(n.total,1)
for (i in 1:n.total) if (fit$fitted.values[i] > 0.5) default[i] <- 1

rdemo <- cbind(rdemo,default)

# compute classification error rates and standard errors

cer.train <- 0
for (i in 1:n.total) {
  if (rdemo$y[i] != rdemo$default[i] && rdemo$partition[i] == 1) {
    cer.train <- cer.train + 1
  }
}
cer.train <- cer.train/n.train

print("classification error rate in training sample and standard error")
print(cer.train)
print(sqrt(cer.train*(1 - cer.train)/n.train))

cer.valid <- 0
for (i in 1:n.total) {
  if (rdemo$y[i] != rdemo$default[i] && rdemo$partition[i] != 1) {
    cer.valid <- cer.valid + 1
  }
}
cer.valid <- cer.valid/n.valid

print("classification error rate in validation sample and standard error")
print(cer.valid)
print(sqrt(cer.valid*(1 - cer.valid)/n.valid))

# run nearest neighbors
print(" ")
print("nearest neighbors")

train.sample <- rdemo[partition==1,1:2]
train.outcome <- rdemo[partition==1,3]
valid.sample <- rdemo[partition==0,1:2]
valid.outcome <- rdemo[partition==0,3]

valid.predict <- knn(train.sample,valid.sample,train.outcome,k=5)

# compute classification error rates and standard errors in validation sample

cer.valid <- 0
for (i in 1:n.valid) {
  if (valid.outcome[i] != valid.predict[i]) {
    cer.valid <- cer.valid + 1
  }
}
cer.valid <- cer.valid/n.valid

print("classification error rate in validation sample and standard error")
print(cer.valid)
print(sqrt(cer.valid*(1 - cer.valid)/n.valid))

# run default tree
print(" ")
print("classification tree")

tree <- rpart(formula = y~x1+x2, data=rdemo, weights=partition, method="class")

# compute classification error rates and standard errors in validation sample

valid.predict <- tree$frame[tree$where,5] - 1
valid.predict <- valid.predict[rdemo$partition==0]

cer.valid <- 0
for (i in 1:n.valid) {
  if (valid.outcome[i] != valid.predict[i]) {
    cer.valid <- cer.valid + 1
  }
}
cer.valid <- cer.valid/n.valid

print("classification error rate in validation sample and standard error")
print(cer.valid)
print(sqrt(cer.valid*(1 - cer.valid)/n.valid))


